-------------------------------------------------------------------------------
-- File name : bit_alu.vhdl
-- Title     : 1-bit ALU
-- project   : ECE 485
-- Library   : single-cycle processor
-- Author(s) : James E. Stine, Jr.
-- Purpose   : definition of single-cycle processor
-- notes :   

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity alu32 is
  port(A    : in std_logic_vector(31 downto 0);
       B    : in std_logic_vector(31 downto 0);
       op   : in std_logic_vector(2 downto 0);
       R    : out std_logic_vector(31 downto 0);
       OvFw : out std_logic;
       Zero : out std_logic);
end alu32;

architecture structure of alu32 is

  component bit_alu
    port(a     : in std_logic;
         b     : in std_logic;
         c_in  : in std_logic;
         less  : in std_logic;
         op    : in std_logic_vector(2 downto 0);
         x     : out std_logic;
         c_out : out std_logic);
  end component;

  component bit_alu_msb
    port(a     : in std_logic;
         b     : in std_logic;
         c_in  : in std_logic;
         less  : in std_logic;
         op    : in std_logic_vector(2 downto 0);
         x     : out std_logic;
         c_out : out std_logic;
         V     : out std_logic;
         set   : out std_logic);
  end component;

  signal zr, set_out : std_logic;
  signal c_out : std_logic_vector(30 downto 0);
  signal res : std_logic_vector(31 downto 0);

begin
	
  zr <= '0';
  bit_alu0  : bit_alu port map(A(0), B(0), op(2), set_out, op, res(0), c_out(0));  
  bit_alu1  : bit_alu port map(A(1), B(1), c_out(0), zr, op, res(1), c_out(1));
  bit_alu2  : bit_alu port map(A(2), B(2), c_out(1), zr, op, res(2), c_out(2));
  bit_alu3  : bit_alu port map(A(3), B(3), c_out(2), zr, op, res(3), c_out(3));
  bit_alu4  : bit_alu port map(A(4), B(4), c_out(3), zr, op, res(4), c_out(4));
  bit_alu5  : bit_alu port map(A(5), B(5), c_out(4), zr, op, res(5), c_out(5));
  bit_alu6  : bit_alu port map(A(6), B(6), c_out(5), zr, op, res(6), c_out(6));
  bit_alu7  : bit_alu port map(A(7), B(7), c_out(6), zr, op, res(7), c_out(7));
  bit_alu8  : bit_alu port map(A(8), B(8), c_out(7), zr, op, res(8), c_out(8));
  bit_alu9  : bit_alu port map(A(9), B(9), c_out(8), zr, op, res(9), c_out(9));
  bit_alu10 : bit_alu port map(A(10), B(10), c_out(9), zr, op, res(10), c_out(10));
  bit_alu11 : bit_alu port map(A(11), B(11), c_out(10), zr, op, res(11), c_out(11));
  bit_alu12 : bit_alu port map(A(12), B(12), c_out(11), zr, op, res(12), c_out(12));
  bit_alu13 : bit_alu port map(A(13), B(13), c_out(12), zr, op, res(13), c_out(13));
  bit_alu14 : bit_alu port map(A(14), B(14), c_out(13), zr, op, res(14), c_out(14));
  bit_alu15 : bit_alu port map(A(15), B(15), c_out(14), zr, op, res(15), c_out(15));
  bit_alu16 : bit_alu port map(A(16), B(16), c_out(15), zr, op, res(16), c_out(16));
  bit_alu17 : bit_alu port map(A(17), B(17), c_out(16), zr, op, res(17), c_out(17));
  bit_alu18 : bit_alu port map(A(18), B(18), c_out(17), zr, op, res(18), c_out(18));
  bit_alu19 : bit_alu port map(A(19), B(19), c_out(18), zr, op, res(19), c_out(19));
  bit_alu20 : bit_alu port map(A(20), B(20), c_out(19), zr, op, res(20), c_out(20));
  bit_alu21 : bit_alu port map(A(21), B(21), c_out(20), zr, op, res(21), c_out(21));
  bit_alu22 : bit_alu port map(A(22), B(22), c_out(21), zr, op, res(22), c_out(22));
  bit_alu23 : bit_alu port map(A(23), B(23), c_out(22), zr, op, res(23), c_out(23));
  bit_alu24 : bit_alu port map(A(24), B(24), c_out(23), zr, op, res(24), c_out(24));
  bit_alu25 : bit_alu port map(A(25), B(25), c_out(24), zr, op, res(25), c_out(25));
  bit_alu26 : bit_alu port map(A(26), B(26), c_out(25), zr, op, res(26), c_out(26));
  bit_alu27 : bit_alu port map(A(27), B(27), c_out(26), zr, op, res(27), c_out(27));
  bit_alu28 : bit_alu port map(A(28), B(28), c_out(27), zr, op, res(28), c_out(28));
  bit_alu29 : bit_alu port map(A(29), B(29), c_out(28), zr, op, res(29), c_out(29));
  bit_alu30 : bit_alu port map(A(30), B(30), c_out(29), zr, op, res(30), c_out(30));
  
  bit_alu_msb31 : bit_alu_msb port map(A(31), B(31), c_out(30), zr, op, res(31), OvFw, set_out);
  
  Zero <= not(res(0) or res(1) or res(2) or res(3) or res(4) or
              res(5) or res(6) or res(7) or res(8) or res(9) or
              res(10) or res(11) or res(12) or res(13) or res(14) or
              res(15) or res(16) or res(17) or res(18) or res(19) or res(20) or
              res(21) or res(22) or res(23) or res(24) or res(25) or res(26) or
              res(27) or res(28) or res(29) or res(30) or res(31));
  R <= res;

end structure;

