-------------------------------------------------------------------------------
-- File name : memory.vhd
-- Title     : Minor Modules of the datapath
-- project   : ECE 485
-- Library   : single-cycle processor
-- Author(s) : James E. Stine, Jr.
-- Purpose   : definition of single-cycle processor
-- notes 	   : Memory modules, instruction mem, data mem, wishbone wrapper

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------


library ieee;
library std;
use ieee.std_logic_1164.ALL;
use std.textio.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity ins_mem512 is
   port(adr     : in  std_logic_vector(8 downto 0);
        memdata : out std_logic_vector(31 downto 0));
end ins_mem512;

architecture struct of ins_mem512 is
begin
  process(adr) is
    file mem_file : text open read_mode is "ins_mem512.dat";  
    variable l : line;
    variable first : boolean := true;
    variable ch : character;
    variable index : integer :=0;
    type ramtype is array (511 downto 0) of std_logic_vector(31 downto 0);
    variable mem : ramtype;
  begin         

    if first then
      first:=false;

      for i in 0 to 511 loop
        mem(conv_integer(i)):="00000000000000000000000000000000";
      end loop;  
      
      while not endfile(mem_file) loop
        readline(mem_file, l);
        for j in 1 to 8 loop
            read(l,ch);
            if  ('0'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0000";
            elsif  ('1'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0001";
            elsif  ('2'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0010";
            elsif  ('3'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0011";
            elsif  ('4'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0100";
            elsif  ('5'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0101";
            elsif  ('6'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0110";
            elsif  ('7'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0111";
            elsif  ('8'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1000";
            elsif  ('9'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1001";
            elsif  ('A'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1010";
            elsif  ('B'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1011";
            elsif  ('C'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1100";
            elsif  ('D'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1101";
            elsif  ('E'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1110";
            elsif  ('F'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1111";
            else
              report "format error on line " & integer'image(index) severity error;
            end if;
        end loop;
        index:=index+1;
      end loop;

    else
        memdata <= mem(conv_integer(adr));
    end if;

  end process;

end struct;


library ieee;
library std;
use ieee.std_logic_1164.all;
use std.textio.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity mem512 is
  port(clk       : in  std_logic;
       memwrite  : in  std_logic;
       adr       : in  std_logic_vector(8 downto 0);
       writedata : in  std_logic_vector(31 downto 0);
       memdata   : out std_logic_vector(31 downto 0));
end mem512;

architecture struct of mem512 is
begin
  process(clk, adr) is
    file mem_file : text open read_mode is "mem512.dat";  
    variable l : line;
    variable first : boolean := true;
    variable ch : character;
    variable index : integer :=0;
    type ramtype is array (511 downto 0) of std_logic_vector(31 downto 0);
    variable mem : ramtype;
  begin         

    if first then
      first:=false;

      for i in 0 to 511 loop
        mem(conv_integer(i)):="00000000000000000000000000000000";
      end loop;  
      
      while not endfile(mem_file) loop
        readline(mem_file, l);
        for j in 1 to 8 loop
            read(l,ch);
            if  ('0'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0000";
            elsif  ('1'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0001";
            elsif  ('2'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0010";
            elsif  ('3'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0011";
            elsif  ('4'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0100";
            elsif  ('5'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0101";
            elsif  ('6'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0110";
            elsif  ('7'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "0111";
            elsif  ('8'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1000";
            elsif  ('9'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1001";
            elsif  ('A'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1010";
            elsif  ('B'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1011";
            elsif  ('C'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1100";
            elsif  ('D'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1101";
            elsif  ('E'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1110";
            elsif  ('F'=ch) then
              mem(index)( ((9-j)*4-1) downto ((9-j)*4-4) ):= "1111";
            else
              report "format error on line " & integer'image(index) severity error;
            end if;
        end loop;   
        index:=index+1;
      end loop;

    else
        memdata <= mem(conv_integer(adr));
        if clk'event and clk='1' and (memwrite='1') then
          mem(conv_integer(adr)):=writedata;
        end if;
    end if;

  end process;

end struct;


library ieee;
library std;
use ieee.std_logic_1164.all;
use std.textio.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity memwb is
  port (ack_o:  out std_logic;
        adr_i:  in  std_logic_vector(  8 downto 0 );
        clk_i:  in  std_logic;
        dat_i:  in  std_logic_vector( 31 downto 0 );
        dat_o:  out std_logic_vector( 31 downto 0 );
        stb_i:  in  std_logic;
        we_i:   in  std_logic);
end memwb;

architecture memwb of memwb is

  component mem512 
    port(clk       : in  std_logic;
         memwrite  : in  std_logic;
         adr       : in  std_logic_vector(8 downto 0);
         writedata : in  std_logic_vector(31 downto 0);
         memdata   : out std_logic_vector(31 downto 0));
  end component;

  signal  lwe : std_logic;

begin

  lwe <= (stb_i and we_i);
  ack_o <= stb_i;

  u00: mem512 port map(writedata => dat_i, adr => adr_i,
                       memwrite => lwe, clk => clk_i, memdata => dat_o);
  
end memwb;
