-------------------------------------------------------------------------------
-- File name : alu32.vhd
-- Title     : 32-bit ALU
-- project   : ECE 485
-- Library   : multi-cycle processor
-- Author(s) : Ivan Castellanos
-- Purpose   : definition of multi-cycle processor
-- notes :   

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity alu32 is
  port (a         : in  std_logic_vector(31 downto 0);
        b         : in  std_logic_vector(31 downto 0);
        operation : in  std_logic_vector(2 downto 0);
        result    : out std_logic_vector(31 downto 0);
        overflow  : out std_logic;
        zero	  : out std_logic);
end alu32;

architecture alu32 of alu32 is

  component bit_alu
    port (a     : in  std_logic;
          b     : in  std_logic;
          c_in  : in  std_logic;
          less  : in  std_logic;
          op    : in  std_logic_vector(2 downto 0);
          x     : out std_logic;
          c_out : out std_logic);
  end component;

  component bit_alu_msb 
    port (a        : in  std_logic;
          b        : in  std_logic;
          c_in     : in  std_logic;
          less     : in  std_logic;
          op       : in  std_logic_vector (2 downto 0);
          x        : out std_logic;
          set      : out std_logic;
          overflow : out std_logic);
  end component;
  
  signal zr, set_out : std_logic;
  signal c_out : std_logic_vector(30 downto 0);
  signal res : std_logic_vector(31 downto 0);

begin
	
  zr <= '0';
  bit_alu0 	: bit_alu port map(a(0), b(0), operation(2), set_out, operation, res( 0), c_out( 0) );  
  bit_alu1	: bit_alu port map(a(1), b(1), c_out(0), zr, operation, res(1), c_out(1));
  bit_alu2	: bit_alu port map(a(2), b(2), c_out(1), zr, operation, res(2), c_out(2));
  bit_alu3	: bit_alu port map(a(3), b(3), c_out(2), zr, operation, res(3), c_out(3));
  bit_alu4	: bit_alu port map(a(4), b(4), c_out(3), zr, operation, res(4), c_out(4));
  bit_alu5	: bit_alu port map(a(5), b(5), c_out(4), zr, operation, res(5), c_out(5));
  bit_alu6	: bit_alu port map(a(6), b(6), c_out(5), zr, operation, res(6), c_out(6));
  bit_alu7	: bit_alu port map(a(7), b(7), c_out(6), zr, operation, res(7), c_out(7));
  bit_alu8	: bit_alu port map(a(8), b(8), c_out(7), zr, operation, res(8), c_out(8));
  bit_alu9	: bit_alu port map(a(9), b(9), c_out(8), zr, operation, res(9), c_out(9));
  bit_alu10	: bit_alu port map(a(10), b(10), c_out(9), zr, operation, res(10), c_out(10));
  bit_alu11	: bit_alu port map(a(11), b(11), c_out(10), zr, operation, res(11), c_out(11));
  bit_alu12	: bit_alu port map(a(12), b(12), c_out(11), zr, operation, res(12), c_out(12));
  bit_alu13	: bit_alu port map(a(13), b(13), c_out(12), zr, operation, res(13), c_out(13));
  bit_alu14	: bit_alu port map(a(14), b(14), c_out(13), zr, operation, res(14), c_out(14));
  bit_alu15	: bit_alu port map(a(15), b(15), c_out(14), zr, operation, res(15), c_out(15));
  bit_alu16	: bit_alu port map(a(16), b(16), c_out(15), zr, operation, res(16), c_out(16));
  bit_alu17	: bit_alu port map(a(17), b(17), c_out(16), zr, operation, res(17), c_out(17));
  bit_alu18	: bit_alu port map(a(18), b(18), c_out(17), zr, operation, res(18), c_out(18));
  bit_alu19	: bit_alu port map(a(19), b(19), c_out(18), zr, operation, res(19), c_out(19));
  bit_alu20	: bit_alu port map(a(20), b(20), c_out(19), zr, operation, res(20), c_out(20));
  bit_alu21	: bit_alu port map(a(21), b(21), c_out(20), zr, operation, res(21), c_out(21));
  bit_alu22	: bit_alu port map(a(22), b(22), c_out(21), zr, operation, res(22), c_out(22));
  bit_alu23	: bit_alu port map(a(23), b(23), c_out(22), zr, operation, res(23), c_out(23));
  bit_alu24	: bit_alu port map(a(24), b(24), c_out(23), zr, operation, res(24), c_out(24));
  bit_alu25	: bit_alu port map(a(25), b(25), c_out(24), zr, operation, res(25), c_out(25));
  bit_alu26	: bit_alu port map(a(26), b(26), c_out(25), zr, operation, res(26), c_out(26));
  bit_alu27	: bit_alu port map(a(27), b(27), c_out(26), zr, operation, res(27), c_out(27));
  bit_alu28	: bit_alu port map(a(28), b(28), c_out(27), zr, operation, res(28), c_out(28));
  bit_alu29	: bit_alu port map(a(29), b(29), c_out(28), zr, operation, res(29), c_out(29));
  bit_alu30	: bit_alu port map(a(30), b(30), c_out(29), zr, operation, res(30), c_out(30));
  
  bit_alu_msb31 : bit_alu_msb port map(a(31), b(31), c_out(30), zr, operation, res(31), set_out, overflow);

  zero <= not(res(0) or res(1) or res(2) or res(3) or res(4) or res(5) or res(6) or res(7) or res(8) or res(9) or
              res(10) or res(11) or res(12) or res(13) or res(14) or res(15) or res(16) or res(17) or
              res(18) or res(19) or res(20) or res(21) or res(22) or res(23) or res(24) or res(25) or
              res(26) or res(27) or res(28) or res(29) or res(30) or res(31));
  result <= res;

end alu32;


