-------------------------------------------------------------------------------
-- File name : alucontrol.vhd
-- Title     : Segmented ALU control
-- project   : ECE 485
-- Library   : multi-cycle processor
-- Author(s) : Ivan Castellanos
-- Purpose   : definition of multi-cycle processor
-- notes :
--
-- bits	 operation
--  000	 and
--  001	 or
--  010	 add
--  011	 (not implemented)
--  100	 xor
--  101	 nor
--  110	 subtract
--  111	 set on less than

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity alucontrol is
  port (aluop        : in  std_logic_vector(1 downto 0);
        opcode_funct : in  std_logic_vector(7 downto 0);
        operation    : out std_logic_vector(2 downto 0));
end alucontrol;

architecture behavior of alucontrol is
begin
  operation(2) <= (aluop(1) and not opcode_funct(7) and not opcode_funct(6)
                   and not opcode_funct(5) and not opcode_funct(4) and opcode_funct(3) and not opcode_funct(2)
                   and opcode_funct(1) and not opcode_funct(0)) or
                  (aluop(1) and not opcode_funct(7)
                   and not opcode_funct(6) and not opcode_funct(5) and not opcode_funct(4) and not opcode_funct(3)
                   and opcode_funct(2) and opcode_funct(1)) or
                  (aluop(1) and not opcode_funct(7)
                   and not opcode_funct(6) and not opcode_funct(5) and not opcode_funct(4) and not opcode_funct(3)
                   and opcode_funct(1) and not opcode_funct(0)) or
                  (aluop(1) and opcode_funct(7)
                   and opcode_funct(5) and not opcode_funct(4)) or
                  (aluop(1) and opcode_funct(7)
                   and not opcode_funct(6) and not opcode_funct(5) and opcode_funct(4)) or
                  (not aluop(1) and aluop(0));

  operation(1) <= (aluop(1) and not opcode_funct(7) and not opcode_funct(6)
                   and not opcode_funct(5) and not opcode_funct(4) and opcode_funct(3) and not opcode_funct(2)
                   and opcode_funct(1) and not opcode_funct(0)) or
                  (not opcode_funct(6)
                   and not opcode_funct(5) and not opcode_funct(4) and not opcode_funct(3) and not opcode_funct(2)
                   and not opcode_funct(0)) or
                  (aluop(1) and opcode_funct(7) and not opcode_funct(6)
                   and not opcode_funct(5) and opcode_funct(4)) or
                  (opcode_funct(7) and not opcode_funct(6)
                   and not opcode_funct(4)) or (not aluop(1));

  operation(0) <= (aluop(1) and not opcode_funct(7) and not opcode_funct(6)
                   and not opcode_funct(5) and not opcode_funct(4) and not opcode_funct(3) and opcode_funct(2)
                   and opcode_funct(0)) or
                  (aluop(1) and not opcode_funct(7) and not opcode_funct(6)
                   and not opcode_funct(5) and not opcode_funct(4) and opcode_funct(3) and not opcode_funct(2)
                   and opcode_funct(1) and not opcode_funct(0)) or
                  (aluop(1) and opcode_funct(7)
                   and opcode_funct(6) and not opcode_funct(5) and opcode_funct(4)) or
                  (aluop(1)
                   and opcode_funct(7) and not opcode_funct(6) and opcode_funct(5) and not opcode_funct(4));

end behavior;


