-------------------------------------------------------------------------------
-- File name : datapath.vhd
-- Title     : Top-level datapath
-- project   : ECE 485
-- Library   : multi-cycle processor
-- Author(s) : Ivan Castellanos
-- Purpose   : definition of multi-cycle processor
-- notes :   

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity datapath is
  port (pcwritecond  : in  std_logic;
        pcwbnq       : in  std_logic;
        pcwrite      : in  std_logic;
        iord         : in  std_logic;
        irwrite      : in  std_logic;
        alusrca      : in  std_logic;
        regwrite     : in  std_logic;
        clk          : in  std_logic;
        causewrite   : in  std_logic;
        intcause     : in  std_logic;
        epcwrite     : in  std_logic;
        aluop        : in  std_logic_vector(1 downto 0);
        alusrcb      : in  std_logic_vector(1 downto 0);
        pcsource     : in  std_logic_vector(1 downto 0);
        regdst       : in  std_logic_vector(1 downto 0);
        memtoreg     : in  std_logic_vector(1 downto 0);
        memdata      : in  std_logic_vector(31 downto 0);
        reset        : in  std_logic;
        cause        : out std_logic;
        overflow     : out std_logic;
        op           : out std_logic_vector(6 downto 0);
        address      : out std_logic_vector(8 downto 0);
        memwritedata : out std_logic_vector(31 downto 0);
        epcout	     : out std_logic_vector(31 downto 0));
end datapath;

architecture datapath of datapath is

  component alu32 
    port (a         : in  std_logic_vector(31 downto 0);
          b         : in  std_logic_vector(31 downto 0);
          operation : in  std_logic_vector(2 downto 0);
          result    : out std_logic_vector(31 downto 0);
          overflow  : out std_logic;
          zero	    : out std_logic);
  end component;

  component alucontrol 
    port (aluop        : in  std_logic_vector(1 downto 0);
          opcode_funct : in  std_logic_vector(7 downto 0);
          operation    : out std_logic_vector(2 downto 0));
  end component;

  component signextend 
    port (a : in  std_logic_vector(15 downto 0);
          y : out std_logic_vector(31 downto 0));
  end component;
  
  component lowzextend 
    port (a : in  std_logic_vector(15 downto 0);
          y : out std_logic_vector(31 downto 0));
  end component;

  component mux21
    port (a   : in  std_logic;
          b   : in  std_logic;
          sel : in  std_logic;
          z   : out std_logic);
  end component;

  component mux21_9
    port (a   : in  std_logic_vector(8 downto 0);
          b   : in  std_logic_vector(8 downto 0);
          sel : in  std_logic;
          z   : out std_logic_vector(8 downto 0));
  end component;

  component mux21_32 
    port (a   : in  std_logic_vector(31 downto 0);
          b   : in  std_logic_vector(31 downto 0);
          sel : in  std_logic;
          z   : out std_logic_vector(31 downto 0));
  end component;

  component mux41_5
    port (a   : in  std_logic_vector(4 downto 0);
          b   : in  std_logic_vector(4 downto 0);
          c   : in  std_logic_vector(4 downto 0);
          sel : in  std_logic_vector(1 downto 0);
          z   : out std_logic_vector(4 downto 0));
  end component;

  component mux41_9 
    port (a   : in  std_logic_vector(8 downto 0);
          b   : in  std_logic_vector(8 downto 0);
          c   : in  std_logic_vector(8 downto 0);
          d   : in  std_logic_vector(8 downto 0);       
          sel : in  std_logic_vector(1 downto 0);
          z   : out std_logic_vector(8 downto 0));
  end component;
  
  component mux41_32 
    port (a   : in  std_logic_vector(31 downto 0);
          b   : in  std_logic_vector(31 downto 0);
          c   : in  std_logic_vector(31 downto 0);
          d   : in  std_logic_vector(31 downto 0);       
          sel : in  std_logic_vector(1 downto 0);
          z   : out std_logic_vector(31 downto 0));  
  end component;

  component dflipflop 
    port (d   : in  std_logic;
          clk : in  std_logic;
          q   : out std_logic);
  end component;

  component register6 
    port (d   : in  std_logic_vector (5 downto 0);
          clk : in  std_logic;
          q   : out std_logic_vector (5 downto 0));
  end component;

  component register9 
    port (d   : in  std_logic_vector (8 downto 0);
          clk : in  std_logic;
          q   : out std_logic_vector (8 downto 0));
  end component;

  component register32 
    port (d   : in  std_logic_vector(31 downto 0);
          clk : in  std_logic;
          q   : out std_logic_vector(31 downto 0));
  end component;

  component registerfile 
    port (readreg1   : in  std_logic_vector(4 downto 0);
          readreg2   : in  std_logic_vector(4 downto 0);
          writereg  : in  std_logic_vector(4 downto 0);
          writedata : in  std_logic_vector(31 downto 0);
          regwrite  : in  std_logic;
          clk       : in  std_logic;
          readdata1 : out std_logic_vector(31 downto 0);
          readdata2 : out std_logic_vector(31 downto 0));
  end component;

  signal  zero, pc_clk, ir_clk, clk_bar, muxint_out, zr1, one1, epcclk, causeclk, overflow_reg, mux_ovf_reset_out : std_logic;
  signal  operation : std_logic_vector(2 downto 0);
  signal  writerf, raaddress : std_logic_vector(4 downto 0);
  signal  op_funct : std_logic_vector(7 downto 0);
  signal  muxaddress_out, pcout, aluout9, aluresult9, jump_addr, intaddr, zr9, muxpcreset_out, mux_address_sel : std_logic_vector(8 downto 0);
  signal  pcout32, aluout, memdreg_out, instruction, write_datarf, lowzexout : std_logic_vector(31 downto 0);
  signal  signout, ain, bin, aout, bout, muxa_out, muxb_out, aluresult, one, zr : std_logic_vector(31 downto 0);
  signal  mux_pc_sel : std_logic_vector (1 downto 0);

begin

  clk_bar <= not clk;
  zr1 <= '0';
  one1 <= '1';
  epcclk <= clk_bar and epcwrite;
  causeclk <= clk_bar and causewrite;
  one(31 downto 0) <= 	"00000000000000000000000000000001";
  zr(31 downto 0) <=  	"00000000000000000000000000000000";
  zr9(8 downto 0) <= "000000000";
  intaddr(8 downto 0) <= "011111110";	-- 0fe hex
  raaddress(4 downto 0) <= "11111";
  pcout32(31 downto 9) <= "00000000000000000000000";
  pcout32(8 downto 0) <= pcout(8 downto 0);
  aluout9(8 downto 0) <= aluout(8 downto 0);
  aluresult9(8 downto 0) <= aluresult (8 downto 0);
  op_funct(7 downto 4) <= instruction(29 downto 26);
  op_funct(3 downto 0) <= instruction(3 downto 0);

  jump_addr(8 downto 0) <= instruction(8 downto 0);

  op(6) <= instruction(31);
  op(5 downto 2) <= instruction(29 downto 26);
  op(1) <= instruction(5);
  op(0) <= instruction(3);

  memwritedata(31 downto 0) <= bout(31 downto 0);

  mux_pc_sel(1) <= not reset;
  mux_pc_sel(0) <= iord;

  pc_clk <= clk_bar and ( pcwrite or (pcwritecond and zero) or (pcwbnq nor zero) );
  ir_clk <= irwrite and not clk;
  
  pc : register9 port map(muxpcreset_out, pc_clk, pcout );
  mux_ovf_reset : mux21 port map('0', overflow_reg, reset, mux_ovf_reset_out);
  muxpcreset : mux21_9 port map(zr9, muxaddress_out, reset, muxpcreset_out);
  muxpc : mux41_9 port map( pcout, aluout9, zr9, zr9, mux_pc_sel, address);
  memdataregister : register32 port map(memdata, clk_bar, memdreg_out);
  instructionregister : register32 port map (memdata, ir_clk, instruction);
  muxwriterf : mux41_5 port map( instruction(20 downto 16), instruction(15 downto 11), raaddress, regdst, writerf);
  muxwrite_datarf : mux41_32 port map( aluout, memdreg_out, pcout32, lowzexout, memtoreg, write_datarf );
  signext : signextend  port map(instruction(15 downto 0) , signout );
  loxext : lowzextend port map(instruction(15 downto 0), lowzexout);
  rf : registerfile port map( instruction(25 downto 21), instruction(20 downto 16), writerf, write_datarf, regwrite, clk, ain, bin );
  rega : register32 port map(ain, clk, aout);
  regb : register32 port map(bin, clk, bout);
  muxa : mux21_32 port map(pcout32, aout, alusrca, muxa_out);
  muxb : mux41_32 port map(bout, one, signout, zr, alusrcb, muxb_out);
  alu : alu32 port map(muxa_out, muxb_out, operation, aluresult, overflow_reg, zero);
  aluctrl : alucontrol port map (aluop, op_funct(7 downto 0), operation);
  aluout_reg : register32 port map (aluresult, clk, aluout);
  muxaddress : mux41_9 port map (aluresult9, aluout9, jump_addr, intaddr, pcsource, muxaddress_out);
  epc : register32 port map(aluresult, epcclk, epcout);
  muxint: mux21 port map(zr1, one1, intcause, muxint_out);
  regcause: dflipflop port map(muxint_out, causeclk, cause);
  regoverflow: dflipflop port map(mux_ovf_reset_out, clk, overflow);

end datapath;


